==> nbgrader-0.6.0.tar.gz: nbgrader-0.6.0/nbgrader/tests/apps/test_nbgrader_releasefeedback.py <==
import os
import sys
from os.path import join, exists, isfile
import pytest

from ...utils import remove, notebook_hash
from .. import run_nbgrader
from .base import BaseTestApp
from .conftest import notwindows


class TestNbGraderReleaseFeedback(BaseTestApp):

    def test_help(self):
        """Does the help display without error?"""
        run_nbgrader(["release_feedback", "--help-all"])

    def test_second_argument(self):
        """Does the help display without error?"""
        run_nbgrader(["release_feedback", "ps1", "second_arg"], retcode=1)

    def test_no_argument(self):
        """Does the help display without error?"""
        run_nbgrader(["release_feedback"], retcode=1)

    @notwindows
    def test_single_file(self, db, course_dir, exchange):
        """Can feedback be generated for an unchanged assignment?"""
        with open("nbgrader_config.py", "a") as fh:
            fh.write("""c.CourseDirectory.db_assignments = [dict(name="ps1")]\n""")
            fh.write("""c.CourseDirectory.db_students = [dict(id="foo")]\n""")
        self._copy_file(join("files", "submitted-unchanged.ipynb"), join(course_dir, "source", "ps1", "p1.ipynb"))
        run_nbgrader(["assign", "ps1", "--db", db])
        nb_path = join(course_dir, "submitted", "foo", "ps1", "p1.ipynb")
        self._copy_file(join("files", "submitted-unchanged.ipynb"), nb_path)
        self._copy_file(join("files", "timestamp.txt"), join(course_dir, "submitted", "foo", "ps1", "timestamp.txt"))
        
        run_nbgrader(["autograde", "ps1", "--db", db])
        run_nbgrader(["generate_feedback", "ps1", "--db", db])
        run_nbgrader(["release_feedback", "ps1", "--Exchange.root={}".format(exchange), '--course', 'abc101'])
        nb_hash = notebook_hash(nb_path)
        assert exists(join(exchange, "abc101", "feedback", "{}.html".format(nb_hash)))
        # release feedback shoul overwrite without error
        run_nbgrader(["release_feedback", "ps1", "--Exchange.root={}".format(exchange), '--course', 'abc101'])

    @notwindows
    def test_single_student(self, db, course_dir, exchange):
        """Can feedback be generated for an unchanged assignment?"""
        with open("nbgrader_config.py", "a") as fh:
            fh.write("""c.CourseDirectory.db_assignments = [dict(name="ps1")]\n""")
            fh.write("""c.CourseDirectory.db_students = [dict(id="foo")]\n""")
        self._copy_file(join("files", "submitted-unchanged.ipynb"), join(course_dir, "source", "ps1", "p1.ipynb"))
        run_nbgrader(["assign", "ps1", "--db", db])
        nb_path = join(course_dir, "submitted", "foo", "ps1", "p1.ipynb")
        self._copy_file(join("files", "submitted-unchanged.ipynb"), nb_path)
        self._copy_file(join("files", "timestamp.txt"), join(course_dir, "submitted", "foo", "ps1", "timestamp.txt"))
        nb_path2 = join(course_dir, "submitted", "bar", "ps1", "p1.ipynb")
        self._copy_file(join("files", "submitted-changed.ipynb"), nb_path2)
        self._copy_file(join("files", "timestamp.txt"), join(course_dir, "submitted", "bar", "ps1", "timestamp.txt"))
        
        run_nbgrader(["autograde", "ps1", "--db", db])
        run_nbgrader(["generate_feedback", "ps1", "--db", db])
        run_nbgrader(["release_feedback", "ps1", "--Exchange.root={}".format(exchange), '--course', 'abc101', '--student', 'foo'])
        nb_hash = notebook_hash(nb_path)
        assert exists(join(exchange, "abc101", "feedback", "{}.html".format(nb_hash)))
        nb_hash2 = notebook_hash(nb_path2)
        assert not exists(join(exchange, "abc101", "feedback", "{}.html".format(nb_hash2)))
        # release feedback shoul overwrite without error
        run_nbgrader(["release_feedback", "ps1", "--Exchange.root={}".format(exchange), '--course', 'abc101'])

    @notwindows
    def test_student_id_exclude(self, db, course_dir, exchange):
        """Does --CourseDirectory.student_id_exclude=X exclude students?"""
        with open("nbgrader_config.py", "a") as fh:
            fh.write("""c.CourseDirectory.db_assignments = [dict(name="ps1")]\n""")
            fh.write("""c.CourseDirectory.db_students = [dict(id="foo"), dict(id="bar"), dict(id="baz")]\n""")
        self._copy_file(join("files", "submitted-unchanged.ipynb"), join(course_dir, "source", "ps1", "p1.ipynb"))
        run_nbgrader(["assign", "ps1", "--db", db])
        nb_path = join(course_dir, "submitted", "foo", "ps1", "p1.ipynb")
        self._copy_file(join("files", "submitted-unchanged.ipynb"), nb_path)
        self._copy_file(join("files", "timestamp.txt"), join(course_dir, "submitted", "foo", "ps1", "timestamp.txt"))
        nb_path2 = join(course_dir, "submitted", "bar", "ps1", "p1.ipynb")
        self._copy_file(join("files", "submitted-changed.ipynb"), nb_path2)
        self._copy_file(join("files", "timestamp.txt"), join(course_dir, "submitted", "bar", "ps1", "timestamp.txt"))

        run_nbgrader(["autograde", "ps1", "--db", db])
        run_nbgrader(["generate_feedback", "ps1", "--db", db])
        run_nbgrader(["release_feedback", "ps1", "--Exchange.root={}".format(exchange), '--course', 'abc101',
                      "--CourseDirectory.student_id_exclude=bar,baz"]) # baz doesn't exist, test still OK though
        nb_hash = notebook_hash(nb_path) # foo
        assert exists(join(exchange, "abc101", "feedback", "{}.html".format(nb_hash)))
        nb_hash2 = notebook_hash(nb_path2) # bar
        assert not exists(join(exchange, "abc101", "feedback", "{}.html".format(nb_hash2)))
        # release feedback should overwrite without error
        run_nbgrader(["release_feedback", "ps1", "--Exchange.root={}".format(exchange), '--course', 'abc101'])


    @notwindows
    @pytest.mark.parametrize("groupshared", [False, True])
    def test_permissions(self, db, course_dir, exchange, groupshared):
        """Are permissions properly set?"""
        with open("nbgrader_config.py", "a") as fh:
            fh.write("""c.CourseDirectory.db_assignments = [dict(name="ps1")]\n""")
            fh.write("""c.CourseDirectory.db_students = [dict(id="foo")]\n""")
            if groupshared:
                fh.write("""c.CourseDirectory.groupshared = True\n""")
        self._copy_file(join("files", "submitted-unchanged.ipynb"), join(course_dir, "source", "ps1", "p1.ipynb"))
        run_nbgrader(["assign", "ps1", "--db", db])
        nb_path = join(course_dir, "submitted", "foo", "ps1", "p1.ipynb")
        self._copy_file(join("files", "submitted-unchanged.ipynb"), nb_path)
        self._copy_file(join("files", "timestamp.txt"), join(course_dir, "submitted", "foo", "ps1", "timestamp.txt"))
        nb_hash = notebook_hash(nb_path)

        self._empty_notebook(join(course_dir, "submitted", "foo", "ps1", "foo.ipynb"))
        run_nbgrader(["autograde", "ps1", "--db", db])
        run_nbgrader(["generate_feedback", "ps1", "--db", db])
        run_nbgrader(["release_feedback", "ps1", "--Exchange.root={}".format(exchange), '--course', 'abc101'])

        if groupshared:
            perms = '664'
            dirperms = '2771'
        else:
            perms = '644'
            dirperms = '711'

        feedback_dir = join(exchange, "abc101", "feedback")
        assert self._get_permissions(feedback_dir) == dirperms
        os.system("find %s -ls"%feedback_dir)
        assert self._get_permissions(join(feedback_dir, nb_hash+".html")) == perms

==> nbgrader-0.6.2.tar.gz: nbgrader-0.6.2/nbgrader/tests/apps/test_nbgrader_releasefeedback.py <==
import os
import sys
from os.path import join, exists, isfile
import pytest

from ...utils import notebook_hash, make_unique_key
from .. import run_nbgrader
from .base import BaseTestApp
from .conftest import notwindows


class TestNbGraderReleaseFeedback(BaseTestApp):

    def test_help(self):
        """Does the help display without error?"""
        run_nbgrader(["release_feedback", "--help-all"])

    def test_second_argument(self):
        """Does the help display without error?"""
        run_nbgrader(["release_feedback", "ps1", "second_arg"], retcode=1)

    def test_no_argument(self):
        """Does the help display without error?"""
        run_nbgrader(["release_feedback"], retcode=1)

    @notwindows
    def test_single_file(self, db, course_dir, exchange):
        """Can feedback be generated for an unchanged assignment?"""
        with open("nbgrader_config.py", "a") as fh:
            fh.write("""c.CourseDirectory.db_assignments = [dict(name="ps1")]\n""")
            fh.write("""c.CourseDirectory.db_students = [dict(id="foo")]\n""")
        self._copy_file(join("files", "submitted-unchanged.ipynb"), join(course_dir, "source", "ps1", "p1.ipynb"))
        run_nbgrader(["assign", "ps1", "--db", db])
        nb_path = join(course_dir, "submitted", "foo", "ps1", "p1.ipynb")
        self._copy_file(join("files", "submitted-unchanged.ipynb"), nb_path)
        self._copy_file(join("files", "timestamp.txt"), join(course_dir, "submitted", "foo", "ps1", "timestamp.txt"))

        run_nbgrader(["autograde", "ps1", "--db", db])
        run_nbgrader(["generate_feedback", "ps1", "--db", db])
        run_nbgrader(["release_feedback", "ps1", "--Exchange.root={}".format(exchange), '--course', 'abc101'])
        unique_key = make_unique_key("abc101", "ps1", "p1", "foo", "2019-05-30 11:44:01.911849 UTC")
        nb_hash = notebook_hash(nb_path, unique_key)
        assert exists(join(exchange, "abc101", "feedback", "{}.html".format(nb_hash)))
        # release feedback should overwrite without error
        run_nbgrader(["release_feedback", "ps1", "--Exchange.root={}".format(exchange), '--course', 'abc101'])

    @notwindows
    def test_single_student(self, db, course_dir, exchange):
        """Can feedback be generated for an unchanged assignment?"""
        with open("nbgrader_config.py", "a") as fh:
            fh.write("""c.CourseDirectory.db_assignments = [dict(name="ps1")]\n""")
            fh.write("""c.CourseDirectory.db_students = [dict(id="foo")]\n""")
        self._copy_file(join("files", "submitted-unchanged.ipynb"), join(course_dir, "source", "ps1", "p1.ipynb"))
        run_nbgrader(["assign", "ps1", "--db", db])
        nb_path = join(course_dir, "submitted", "foo", "ps1", "p1.ipynb")
        self._copy_file(join("files", "submitted-unchanged.ipynb"), nb_path)
        self._copy_file(join("files", "timestamp.txt"), join(course_dir, "submitted", "foo", "ps1", "timestamp.txt"))
        nb_path2 = join(course_dir, "submitted", "bar", "ps1", "p1.ipynb")
        self._copy_file(join("files", "submitted-changed.ipynb"), nb_path2)
        self._copy_file(join("files", "timestamp.txt"), join(course_dir, "submitted", "bar", "ps1", "timestamp.txt"))

        run_nbgrader(["autograde", "ps1", "--db", db])
        run_nbgrader(["generate_feedback", "ps1", "--db", db])
        run_nbgrader(["release_feedback", "ps1", "--Exchange.root={}".format(exchange), '--course', 'abc101', '--student', 'foo'])
        unique_key = make_unique_key("abc101", "ps1", "p1", "foo", "2019-05-30 11:44:01.911849 UTC")
        nb_hash = notebook_hash(nb_path, unique_key)
        assert exists(join(exchange, "abc101", "feedback", "{}.html".format(nb_hash)))
        unique_key2 = make_unique_key("abc101", "ps1", "p1", "bar", "2019-05-30 11:44:01.911849 UTC")
        nb_hash2 = notebook_hash(nb_path2, unique_key2)
        assert not exists(join(exchange, "abc101", "feedback", "{}.html".format(nb_hash2)))
        # release feedback should overwrite without error
        run_nbgrader(["release_feedback", "ps1", "--Exchange.root={}".format(exchange), '--course', 'abc101'])

    @notwindows
    def test_student_id_exclude(self, db, course_dir, exchange):
        """Does --CourseDirectory.student_id_exclude=X exclude students?"""
        with open("nbgrader_config.py", "a") as fh:
            fh.write("""c.CourseDirectory.db_assignments = [dict(name="ps1")]\n""")
            fh.write("""c.CourseDirectory.db_students = [dict(id="foo"), dict(id="bar"), dict(id="baz")]\n""")
        self._copy_file(join("files", "submitted-unchanged.ipynb"), join(course_dir, "source", "ps1", "p1.ipynb"))
        run_nbgrader(["assign", "ps1", "--db", db])
        nb_path = join(course_dir, "submitted", "foo", "ps1", "p1.ipynb")
        self._copy_file(join("files", "submitted-unchanged.ipynb"), nb_path)
        self._copy_file(join("files", "timestamp.txt"), join(course_dir, "submitted", "foo", "ps1", "timestamp.txt"))
        nb_path2 = join(course_dir, "submitted", "bar", "ps1", "p1.ipynb")
        self._copy_file(join("files", "submitted-changed.ipynb"), nb_path2)
        self._copy_file(join("files", "timestamp.txt"), join(course_dir, "submitted", "bar", "ps1", "timestamp.txt"))

        run_nbgrader(["autograde", "ps1", "--db", db])
        run_nbgrader(["generate_feedback", "ps1", "--db", db])
        run_nbgrader(["release_feedback", "ps1", "--Exchange.root={}".format(exchange), '--course', 'abc101',
                      "--CourseDirectory.student_id_exclude=bar,baz"]) # baz doesn't exist, test still OK though
        unique_key = make_unique_key("abc101", "ps1", "p1", "foo", "2019-05-30 11:44:01.911849 UTC")
        nb_hash = notebook_hash(nb_path, unique_key) # foo
        assert exists(join(exchange, "abc101", "feedback", "{}.html".format(nb_hash)))
        unique_key2 = make_unique_key("abc101", "ps1", "p1", "bar", "2019-05-30 11:44:01.911849 UTC")
        nb_hash2 = notebook_hash(nb_path2, unique_key2) # bar
        assert not exists(join(exchange, "abc101", "feedback", "{}.html".format(nb_hash2)))
        # release feedback should overwrite without error
        run_nbgrader(["release_feedback", "ps1", "--Exchange.root={}".format(exchange), '--course', 'abc101'])


    @notwindows
    @pytest.mark.parametrize("groupshared", [False, True])
    def test_permissions(self, db, course_dir, exchange, groupshared):
        """Are permissions properly set?"""
        with open("nbgrader_config.py", "a") as fh:
            fh.write("""c.CourseDirectory.db_assignments = [dict(name="ps1")]\n""")
            fh.write("""c.CourseDirectory.db_students = [dict(id="foo")]\n""")
            if groupshared:
                fh.write("""c.CourseDirectory.groupshared = True\n""")
        self._copy_file(join("files", "submitted-unchanged.ipynb"), join(course_dir, "source", "ps1", "p1.ipynb"))
        run_nbgrader(["assign", "ps1", "--db", db])
        nb_path = join(course_dir, "submitted", "foo", "ps1", "p1.ipynb")
        self._copy_file(join("files", "submitted-unchanged.ipynb"), nb_path)
        self._copy_file(join("files", "timestamp.txt"), join(course_dir, "submitted", "foo", "ps1", "timestamp.txt"))
        unique_key = make_unique_key("abc101", "ps1", "p1", "foo", "2019-05-30 11:44:01.911849 UTC")
        nb_hash = notebook_hash(nb_path, unique_key)

        self._empty_notebook(join(course_dir, "submitted", "foo", "ps1", "foo.ipynb"))
        run_nbgrader(["autograde", "ps1", "--db", db])
        run_nbgrader(["generate_feedback", "ps1", "--db", db])
        run_nbgrader(["release_feedback", "ps1", "--Exchange.root={}".format(exchange), '--course', 'abc101'])

        if groupshared:
            perms = '664'
            dirperms = '2771'
        else:
            perms = '644'
            dirperms = '711'

        feedback_dir = join(exchange, "abc101", "feedback")
        assert self._get_permissions(feedback_dir) == dirperms
        os.system("find %s -ls"%feedback_dir)
        assert self._get_permissions(join(feedback_dir, nb_hash+".html")) == perms
